﻿using System;
using System.Collections.ObjectModel;
using System.Data;
using System.IO;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Threading;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace post7923
{
    public partial class Window1 : Window
    {
        private double previousLong;
        private double previousLat;
        private Marker vehicle;
        private DispatcherTimer timer;
        private Collection<AreaBaseShape> zones;
        private StreamReader mGPSData;
        private Storyboard storyboard;

        public Window1()
        {
            InitializeComponent();

            mGPSData = new StreamReader(@"..\..\data\GPSinfo.txt");
            storyboard = (Storyboard)Resources["Storyboard1"];

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromMilliseconds(500);
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();

            vehicle = new Marker(-97.7510219100952, 30.3058518333435);
            vehicle.ImageSource = new BitmapImage(new Uri("/Images/Sedan.png", UriKind.RelativeOrAbsolute));
            vehicle.RenderTransform = new RotateTransform(37, 25, 25);
        }

        private void map_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;

            //Creates the polygons for the prohibited zones.
            PolygonShape polygonShape = new PolygonShape();
            polygonShape.OuterRing.Vertices.Add(new Vertex(-97.7406, 30.3057));
            polygonShape.OuterRing.Vertices.Add(new Vertex(-97.7354, 30.3031));
            polygonShape.OuterRing.Vertices.Add(new Vertex(-97.7384, 30.2979));
            polygonShape.OuterRing.Vertices.Add(new Vertex(-97.7445, 30.3006));
            polygonShape.OuterRing.Vertices.Add(new Vertex(-97.7406, 30.3057));

            PolygonShape polygonShape2 = new PolygonShape();
            polygonShape2.OuterRing.Vertices.Add(new Vertex(-97.747, 30.3086));
            polygonShape2.OuterRing.Vertices.Add(new Vertex(-97.745, 30.3078));
            polygonShape2.OuterRing.Vertices.Add(new Vertex(-97.7463, 30.3057));
            polygonShape2.OuterRing.Vertices.Add(new Vertex(-97.7486, 30.3066));
            polygonShape2.OuterRing.Vertices.Add(new Vertex(-97.747, 30.3086));

            zones = new Collection<AreaBaseShape>();
            zones.Add(polygonShape);
            zones.Add(polygonShape2);

            //InMemoryFeatureLayer for zones.
            InMemoryFeatureLayer zoneInMemoryFeatureLayer = new InMemoryFeatureLayer();
            zoneInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(80, GeoColor.StandardColors.Red), GeoColor.StandardColors.Black);
            zoneInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            zoneInMemoryFeatureLayer.InternalFeatures.Add("Zone1", new Feature(polygonShape));
            zoneInMemoryFeatureLayer.InternalFeatures.Add("Zone2", new Feature(polygonShape2));

            WorldMapKitWmsWpfOverlay wmkOverlay = new WorldMapKitWmsWpfOverlay();
            wpfMap1.Overlays.Add(wmkOverlay);

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add(zoneInMemoryFeatureLayer);
            wpfMap1.Overlays.Add(overlay);

            SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay();
            markerOverlay.Markers.Add(vehicle);
            wpfMap1.Overlays.Add(markerOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-97.7591, 30.3126, -97.7317, 30.2964);
            wpfMap1.Refresh();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            //Gets the GPS info from the textfile.
            DataTable carData = GetCarData();

            // Get the Row of Data we are working with.
            DataRow carDataRow = carData.Rows[0];

            double Lat = Convert.ToDouble(carDataRow["LAT"]);
            double Long = Convert.ToDouble(carDataRow["LONG"]);

            double angle = GetAngleFromTwoVertices(new Vertex(previousLong, previousLat), new Vertex(Long, Lat));

            vehicle.Position = new Point(Long, Lat);
            vehicle.RenderTransform = new RotateTransform(angle - 90, 25, 25);

            //At each new vehicle location, it checks if the vehicle is inside one of the zones.
            //If it is, it enables flashingTimer.
            bool isInside = false;
            PointShape pointShape = new PointShape(Long, Lat);
            foreach (AreaBaseShape areaBaseShape in zones)
            {
                if (areaBaseShape.Contains(pointShape))
                {
                    isInside = true;
                    break;
                }
            }

            if (isInside)
            {
                storyboard.FillBehavior = FillBehavior.Stop;
                Storyboard.SetTarget(storyboard.Children[0], vehicle);
                Storyboard.SetTargetProperty(storyboard.Children[0], new PropertyPath(Marker.OpacityProperty));
                storyboard.Begin();
            }

            previousLat = Lat;
            previousLong = Long;
        }

        private DataTable GetCarData()
        {
            DataTable datatable = new DataTable();
            datatable.Columns.Add("LAT");
            datatable.Columns.Add("LONG");

            string strLattitude = "";
            string strLongitude = "";

            // Read the next line from the text file with GPS data in it.
            string strCurrentText = mGPSData.ReadLine();

            if (strCurrentText == "")
            {
                mGPSData.BaseStream.Seek(0, SeekOrigin.Begin);
                strCurrentText = mGPSData.ReadLine();
            }

            while (strCurrentText != null)
            {
                // Every other line is a "/" and we want to skip those.
                if (strCurrentText.Trim() != "/")
                {
                    string[] strSplit = strCurrentText.Split(','); // (':');
                    strLongitude = strSplit[0];
                    strLattitude = strSplit[1];
                    break;
                }
                strCurrentText = mGPSData.ReadLine();
            }

            object[] objs = new object[2] { strLattitude, strLongitude };

            datatable.Rows.Add(objs);

            return datatable;
        }

        private double GetAngleFromTwoVertices(Vertex b, Vertex c)
        {
            double alpha = 0;
            double tangentAlpha = (c.Y - b.Y) / (c.X - b.X);
            double Peta = Math.Atan(tangentAlpha);

            if (c.X > b.X)
            {
                alpha = 90 - (Peta * (180 / Math.PI));
            }
            else if (c.X < b.X)
            {
                alpha = 270 - (Peta * (180 / Math.PI));
            }
            else
            {
                if (c.Y > b.Y) alpha = 0;
                if (c.Y < b.Y) alpha = 180;
            }
            return alpha;
        }
    }
}
